import QtQuick 2.2
import QtQuick.Window 2.2

import AdaptDemoSystem 1.0

import "./_SharedQML"

Scene {
  id: scene
  property string name: "skene"
  property string sceneName: ""
  property string sceneDir: "fx\\"+sceneName+"\\"
  property bool bSceneEnabled: true // syncRoot("ON")


  Rocket {
    id: rocket

    onTrackFocused: {
    //  console.log("RocketScene: trackFocused from rocket:"+trackName+","+trackValue)
      root.rocketTrackFocused(trackName, trackValue);
    }

  }

  onSetSceneName: {
    sceneName = sn;
    name = sn;
  }


  property int frameInd: 0
  Connections {
    target: demo; onFrameRendered: {
      frameInd++;
    }
  }

  property real rocket_bpm: 128.0


  Component {
    id: localOscComp
    OscClock {
      reset: (name !== "") ? syncTrigger(name+".reset")+fftReset : 0.0
      clockSpeed: (name !== "") ? (syncFunc(name+".freq")+fftToFreq*fftValue)*60.0/rocket_bpm : 0.0
      resetTime: (name !== "") ? syncFunc(name+".reset") : 0.0

      property var syncFunc

      property real fftReset: 0.0
      property bool evalHasFft: false // (name !== "") ? hasFft(syncFunc(name+"_fft")) : false
      property real fftValue: 0.0 // (name !== "") ? getFft(syncFunc(name+"_fft")) : 0.0
      property real fftThr: 0.0 // ((name !== "") && evalHasFft) ? syncFunc(name+"_fft.resetThr") : 0.0
      property real fftGain: 0.0 // ((name !== "") && evalHasFft) ? syncFunc(name+"_fft.resetGain") : 0.0
      property real fftToFreq: 0.0 // ((name !== "") && evalHasFft) ? syncFunc(name+"_fft.FM") : 0.0

      property real valueScale: 1.0
      property real valueAccu: 0.0
      property real accu: 0.0

      property real resetAccu: (name !== "") ? syncTrigger(name+".resetAccu") : 0.0
      onResetAccuChanged: {
        accu = resetAccu;
      }

      onResetDone: {
        //  console.log("osc:"+name, "osc reset, valueAccu:"+valueAccu, "accu:"+accu)
        accu += valueAccu;
        valueAccu = 0.0;
      }

      onFftValueChanged: {
        if ((fftValue > fftThr) && (time >= 1.0) && (fftThr>0.01)) {
          fftReset += 1.0;
          valueScale = fftValue*fftGain+1.0;
        }
      }
    }
  }

  Component {
    id: localFftComp
    FFT {
      property var syncFunc
      areaWidth: (name !== "") ? syncFunc(name+".width")*0.5 : 0.0
      freq: (name !== "") ? syncFunc(name+".freq") : 0.0
      smooth: (name !== "") ? Math.pow(0.5, 1.0/(syncFunc(name+".split")*60.0)) : 0.0
      smoothAttack: (name !== "") ? Math.pow(0.5, 1.0/(syncFunc(name+".splitAttack")*60.0)) : 0.0
    }
  }

  property string fftRoot: "fx/fftData/"
  property var fftPresets
  property var fftPresetsMap

  Filer {
    id: filer
  }

  function loadFftPresets() {
    if (fftPresets !== undefined) {
      console.log("fftPresets loaded ok");
      console.log("fftPresets.presets.length:"+fftPresets.presets.length);
      return;
    }

    var p = filer.getFileText(fftRoot+"presets.json");
    console.log("loading fft presets from file:"+fftRoot+"presets.json");
    if (p !== "") {
      fftPresets = JSON.parse(p);
      fftPresetsMap = {}
      for (var i=0; i<fftPresets.presets.length; i++) {
        fftPresetsMap[fftPresets.presets[i].index] = fftPresets.presets[i].name;
      }
    } else {
      console.log("file contents empty");
    }

  }

  Component.onCompleted: {
    loadFftPresets();
  }




  Component {
    id: localFftImageComp
    FFTImage {
      property int index: 0
      id: fftImg

//      property Texture tex: fftTex
//      Texture {
//        id: fftTex
//        texImage: fftImg.texImage
//      }

      fftPreset: {
        loadFftPresets();
        if (fftPresets === undefined) {
          return "";
        }
        if (fftPresetsMap[index] === undefined) {
          return "";
        }
        return fftPresetsMap[index];
      }
    }
  }

  Component {
    id: localNoiseComp
    NoiseGenerator {
      id: ngen
      freq: (name !== "") ? (syncRoot(ngen.name+".freq")+ngen.fftToFreq*ngen.fftValue)*60.0/rocket_bpm : 0.0
      seed: (name !== "") ? syncTrigger(name+".reset") : 0.0
      interpolation: (name !== "") ? syncRoot(name+".interpolation") : 0.0

      property real fftReset: 0.0
      property bool evalHasFft: false // (name !== "") ? hasFft(syncRoot(name+"_fft")) : false
      property real fftValue: 0.0 // (name !== "") ? getFft(syncRoot(name+"_fft")) : 0.0
      property real fftThr: 0.0 // ((name !== "") && evalHasFft) ? syncRoot(name+"_fft.resetThr") : 0.0
      property real fftGain: 0.0 // ((name !== "") && evalHasFft) ? syncRoot(name+"_fft.resetGain") : 0.0
      property real fftToFreq: 0.0 // ((name !== "") && evalHasFft) ? syncRoot(name+"_fft.FM") : 0.0

      property real time: clock.time
      OscClock {
        id: clock
        reset: (ngen.name !== "") ? syncTrigger(ngen.name+".reset")+ngen.fftReset : 0.0
        clockSpeed: (ngen.name !== "") ? (syncRoot(ngen.name+".freq")+ngen.fftToFreq*ngen.fftValue)*60.0/rocket_bpm : 0.0
        resetTime: 0.0

        onResetDone: {
          //  console.log("osc:"+name, "osc reset, valueAccu:"+valueAccu, "accu:"+accu)
          ngen.accu += ngen.valueAccu;
          ngen.valueAccu = 0.0;
        }
      }

      property real valueScale: 1.0
      property real valueAccu: 0.0
      property real accu: 0.0

      property real resetAccu: (name !== "") ? syncTrigger(name+".resetAccu") : 0.0
      onResetAccuChanged: {
        accu = resetAccu;
      }

      onFftValueChanged: {
        if ((fftValue > fftThr) && (time >= 1.0) && (fftThr>0.01)) {
          fftReset += 1.0;
          valueScale = fftValue*fftGain+1.0;
        }
      }
    }
  }

  function getSceneDir() {
    return sceneDir
  }

  property var localOscs: []
  property var localFfts: []

  function smoothstep(edge0, edge1, x) {
    x = clamp((x-edge0)/(edge1-edge0), 0.0, 1.0);
    return x*x*(3.0-2.0*x);
  }

  function clamp(x, lowerlimit, upperlimit) {
    if (x < lowerlimit)
      x = lowerlimit;
    if (x > upperlimit)
      x = upperlimit;
    return x;
  }

  function syncOscBase(track, oscName, defaultValue, syncFunc) {
    if (sceneName == "") {
      console.log("sync called before init done");
      return 1.0;
    } else {
      var rv = syncFunc(track, defaultValue);
      var oscNum = syncFunc(track+"."+oscName+".num");
      if (oscNum > 0) {
        oscNum = Math.min(oscNum, 3);
        for (var oscIndex=0; oscIndex < oscNum; oscIndex++) {
          var oscNameNum = oscName+oscIndex;
          var oscInd = syncFunc(track+"."+oscNameNum+".global");
          if (((oscInd < 0) || (oscInd > 0)) && (oscInd<4)) {
            var oscAmp = syncFunc(track+"."+oscNameNum+".amp");
            var oscMode = syncFunc(track+"."+oscNameNum+".mode");
            if (oscInd > 0) {
              oscNameNum = "_osc"+oscInd;
            }

            var localId;
            localId = track+oscNameNum;

            if (oscMode < 9.999) { // regular oscillator
              localId = track+oscNameNum;
              if (localOscs[localId] === undefined) {
                localOscs[localId] = localOscComp.createObject(scene);
                localOscs[localId].syncFunc = syncFunc;
              }
            } else if (oscMode < 19.999) { // noise oscillator
              localId = track+oscNameNum+"noise";
              if (localOscs[localId] === undefined) {
                localOscs[localId] = localNoiseComp.createObject(scene);
                // localOscs[localId].syncFunc = syncFunc;
              }
            }

            if (oscInd > 0) {
              // global
              localOscs[localId].name = "_osc"+oscInd;
            } else {
              localOscs[localId].name = track+"."+oscNameNum+"local";
            }

            var subMode = (oscMode-oscMode.toFixed(0))*10.0;
            var doAccu = false;
            if (subMode < 0.9) {
              doAccu = false;
            } else {
              doAccu = true;
            }

            oscMode = oscMode.toFixed();

            var osc = localOscs[localId];

            oscAmp *= osc.valueScale;

            var v = 0.0;

            if (oscMode < 10) { // modes up to 9 reserved for osc
              //                            switch (oscMode) {
              //                                case 0: // mode 0: sine wave
              //                                    v = Math.sin(osc.time*2.0*Math.PI);
              //                                break;
              //                                case 1: // mode 1: smooth step
              //                                    v = smoothstep(0.0, 1.0, osc.time);
              //                                break;
              //                                case 2:
              //                                    v = (smoothstep(0.0, 0.50, osc.time))*(smoothstep(0.0, 0.50, 1.0-osc.time));
              //                                break;
              //                                case 3:
              //                                    v = (smoothstep(0.0, 0.50, osc.time))*(smoothstep(0.0, 0.50, 1.0-osc.time))*Math.sin(osc.time);
              //                                break;
              //                                case 4: // saw up
              //                                    v = Math.fract(osc.time);
              //                                break;
              //                                case 5: // saw down
              //                                    v = 1.0-Math.fract(osc.time);
              //                                break;
              //                                case 6: // triangle
              //                                    var ph = Math.fract(osc.time);
              //                                    if (ph < 0.5) {
              //                                        v = ph*2.0;
              //                                    } else {
              //                                        v = 1.0-(ph-0.5)*2.0;
              //                                    }
              //                                break;
              //                            }
              // v = Math.sin(osc.time*2.0*Math.PI);

              if (oscMode < 0.999) { // mode 0: sine wave
                v = Math.sin(osc.time*2.0*Math.PI);
              } else if (oscMode < 1.999) { // mode 1: smooth step
                v = smoothstep(0.0, 1.0, osc.time);
              } else if (oscMode < 2.999) { // mode 2: smooth "bell" (step in & out)
                v = (smoothstep(0.0, 0.50, osc.time))*(smoothstep(0.0, 0.50, 1.0-osc.time));
              } else if (oscMode < 3.999) { // mode 3: smooth "bell" (step in & out) with sine
                v = (smoothstep(0.0, 0.50, osc.time))*(smoothstep(0.0, 0.50, 1.0-osc.time))*Math.sin(osc.time*2.0*Math.PI);
              } else if (oscMode < 4.999) { // mode 4: saw up
                v = (osc.time-Math.floor(osc.time))*2.0-1.0;
              } else if (oscMode < 5.999) { // mode 5: saw down
                v = (1.0-(osc.time-Math.floor(osc.time)))*2.0-1.0;
              } else if (oscMode < 6.999) { // mode 6: triangle
                var ph = osc.time-Math.floor(osc.time);
                if (ph < 0.5) {
                  v = ph*2.0;
                } else {
                  v = 1.0-(ph-0.5)*2.0;
                }
                v = v*2.0-1.0;
              } else if (oscMode < 7.999) { // mode 7: cosine wave
                v = Math.cos(osc.time*2.0*Math.PI);
              }
              v *= oscAmp;

            } else if (oscMode < 20) { // modes from 10 to 19 reserved for noise oscs
              if (oscMode < 10.999) { // mode 10: noise gen
                v = oscAmp*osc.value;
              }
            }
            rv += v+osc.accu;
            if (doAccu && (osc.time>=0.001)) {
              osc.valueAccu = v;
            }
          }
        }
      }
      return rv;
    }
  }


  function syncFFTBaseImage(track, fftName, defaultValue, syncFunc) {
    if (sceneName == "") {
      console.log("sync called before init done");
      return 1.0;
    } else {

      var rv = syncFunc(track, defaultValue);
      var fftInd = syncFunc(track+".fftInd", 0.0).toFixed(0);
      var fftAmp = syncFunc(track+".fftAmp", 0.0);

      if (fftInd > 0) {
        var localId = track+fftName+fftInd;
        if (localFfts[localId] === undefined) {
          localFfts[localId] = localFftImageComp.createObject(scene);
         // localFfts[localId].syncFunc = syncFunc;
          console.log("create localFFT with id:"+localId)
        }
       // console.log("fftAmp:"+fftAmp);
        var fft = localFfts[localId];
        fft.index = fftInd;
        //rv += fftAmp*fft.fftImage.value;
        rv += fftAmp*fft.value;

       // console.log("local fft sync to:"+rv);

        //                        var fftThr = 0.0; // syncFunc(localName+".thr");
        //                        var fftExp = 1.0; // syncFunc(localName+".exp");
        //                        rv += fftAmp*Math.pow(Math.max(fft.getCurrentValue()-fftThr, 0.0), 1.0+fftExp);
      }
      return rv;
    }
  }

  function getFFTBaseTexImage(track, fftName, syncFunc) {
    var fftInd = syncFunc(track+".fftInd", 0.0).toFixed(0);
    if (fftInd > 0) {
      var localId = track+fftName+fftInd;
      if (localFfts[localId] !== undefined) {
        var fft = localFfts[localId];
        var rv = 0.0;
        rv += fft.value;
        return fft.texImage;
      }
    }
  }


  function syncOsc(track, defaultValue) {
    return syncOscBase(track, "-osc", defaultValue, syncRoot);
  }
  function syncFFT(track, defaultValue) {
    //   return syncFFTBase(track, "-fft", defaultValue, syncRoot);
    //   return syncRoot(track);

    return syncFFTBaseImage(track, "-fft", defaultValue, syncRoot);
  }
  function getTexImageFFT(track) {
    return getFFTBaseTexImage(track, "-fft", syncRoot)
  }


  function syncOscFFT(track, defaultValue) {
    return syncOscBase(track, "-osc", defaultValue, syncRoot)+syncFFTBaseImage(track, "-fft", defaultValue, syncRoot)-syncRoot(track, defaultValue);
  }

  function syncRoot(track, defaultValue) {
    if (sceneName == "") {
      // not yet finished initializing the skenes...
      console.log("sync called before init done");
      return 1.0;
    } else {
      root.addEffectTrack("", track, defaultValue)

      return syncRocketRoot(sceneName+"."+track, defaultValue)
      //   return syncRocketRoot(sceneName+"."+track)
    }
  }

  function syncRootOn(track, effectType, defaultValue) {
    if (sceneName == "") {
      // not yet finished initializing the skenes...
      console.log("sync called before init done");
      return 1.0;
    } else {
      if (root.fxInstancesPerType === undefined) {
        root.fxInstancesPerType = {};
      }
      if (effectType !== undefined && effectType !== "") {
        if (root.fxInstancesPerType[effectType] === undefined) {
          root.fxInstancesPerType[effectType] = {};
        }
        var effectInst = track;
        root.fxInstancesPerType[effectType][effectInst] = true;
      }
      if (defaultValue === undefined) {
        defaultValue = 0.0;
      }

      root.addEffectTrack(effectType, track+".ON")

      return syncRocketRoot(sceneName+"."+track+".ON", defaultValue)
      //   return syncRocketRoot(sceneName+"."+track)
    }
  }

  function syncTrigger(track) {
    if (sceneName == "") {
      // console.log("ERROR: specify \"sceneName\" for RocketScene");
      // not yet finished initializing the skenes...
      console.log("sync called before init done");
      return 1.0;
    } else {
      return syncTriggerRocketRoot(sceneName+"."+track)
      //return syncRocketRoot(track)
      //     return 1.0;
    }
  }

  function blendMode(rn) {
    var v = syncRoot(rn+".blendMode");
    return v;
  }

  function blendSelBase(v) {
    if (v===0) return "off";
    if (v===1) return "normal";
    if (v===2) return "add";
    if (v===3) return "mul";
    if (v===4) return "sub";
    return "off";
  }

  function blendSel(rn) {
    var v = syncRoot(rn+".blendMode", 0.0);
    return blendSelBase(v);
  }

  function blendSelPresets(rn, presets) {
    var v = presets.get("blendMode");
    return blendSelBase(v);
  }

  MeshList {
    id: meshList
  }

  function meshSelBase(v) {
    //  var v = syncRoot(rn+".meshIndex", 18).toFixed(0);

    if (v < 1000) {
      v = Math.min(Math.max(0, v), meshList.content0.length-1);
      return "mesh/"+meshList.content0[v];
    }
    if (v < 2000) {
      v-=1000;
      v = Math.min(Math.max(0, v), meshList.content1.length-1);
      return "mesh/"+meshList.content1[v];
    }
    if (v < 3000) {
      v-=2000;
      v = Math.min(Math.max(0, v), meshList.content2.length-1);
      return "mesh/"+meshList.content2[v];
    }
    if (v < 4000) {
      v-=3000;
      v = Math.min(Math.max(0, v), meshList.content3.length-1);
      return "mesh/"+meshList.content3[v];
    }
    if (v < 5000) {
      v-=4000;
      v = Math.min(Math.max(0, v), meshList.content4.length-1);
      return "mesh/"+meshList.content4[v];
    }
    if (v < 6000) {
      v-=5000;
      v = Math.min(Math.max(0, v), meshList.content5.length-1);
      return "mesh/"+meshList.content5[v];
    }
    if (v < 10000) {
      v-=9000;
      v = Math.min(v, meshList.content9.length-1);
      return "mesh/"+meshList.content9[v];
    }

  }

  function meshSel(rn) {
    var v = syncRoot(rn+".meshIndex", 0).toFixed(0);
    return meshSelBase(v);
  }

  function meshSel2(rn) {
    var v = syncRoot(rn+".meshIndex2", 0).toFixed(0);
    if (v<1000) {
      return "";
    }
    return meshSelBase(v);
  }

  function meshSelPresets(rn, presets) {
    var v = presets.get("meshIndex").toFixed(0);
    return meshSelBase(v);
  }

  function meshSelPrefix(rn, prefix) {
    var v = syncRoot(rn+".meshIndex").toFixed(0);
    v = Math.min(Math.max(0, v), meshList.content.length-1);
    return prefix+meshList.content[v];
  }

  sceneEnabled: bSceneEnabled
  enabled: bSceneEnabled
  visible: bSceneEnabled

}

